<?php

namespace Drupal\drupal_sheetjs\EventSubscriber;

use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\drupal_sheetjs\Services\DrupalSheetjsConfigService;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event subscriber for Drupal SheetJS functionality.
 *
 * @package Drupal\drupal_sheetjs\EventSubscriber
 */
class DrupalSheetjsEventSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  private $currentUser;

  /**
   * The URL generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a DrupalSheetjsEventSubscriber object.
   *
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   The URL generator.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   */
  public function __construct(AccountInterface $currentUser, UrlGeneratorInterface $urlGenerator, RendererInterface $renderer) {
    $this->currentUser = $currentUser;
    $this->urlGenerator = $urlGenerator;
    $this->renderer = $renderer;
  }

  /**
   * Handles the kernel response event.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The response event.
   */
  public function onKernelResponse(FilterResponseEvent $event) {
    $response = $event->getResponse();
    $request = $event->getRequest();

    // Do not capture redirects or modify XML HTTP Requests.
    if ($request->isXmlHttpRequest()) {
      return;
    }

    if ($this->currentUser->hasPermission('access drupal sheetjs')) {
      $this->injectDrupalSheetjs($response);
    }
  }

  /**
   * Injects the Drupal SheetJS functionality into the response.
   *
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   The response object.
   */
  protected function injectDrupalSheetjs(Response $response) {
    $content = $response->getContent();
    $pos = mb_strripos($content, '</body>');

    //
    // Data with no pagination for download
    //
    // Data structure sample below:
    //
    // -------------------------------
    // | title01 | title02 | title03 |
    // -------------------------------
    // | data01  | data02  | data03  |
    // | data11  | data12  | data13  |
    // | data21  | data22  | data23  |
    // -------------------------------
    //
    // $full_data = [];.
    if (FALSE !== $pos) {
      $loader = [
        '#theme' => "drupal_sheetjs_loader",
        '#drupal_sheetjs_config' => (new DrupalSheetjsConfigService())->getData(),
      ];
      $content = mb_substr($content, 0, $pos) . $this->renderer->renderRoot($loader) . mb_substr($content, $pos);
      $response->setContent($content);
    }
  }

  /**
   * Retrieves the subscribed events.
   *
   * @return array
   *   An array of subscribed events.
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => ['onKernelResponse', -120],
    ];
  }

}
