<?php

namespace Drupal\Tests\drupal_sheetjs\Browser;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests the backend configuration page.
 *
 * @package Drupal\Tests\drupal_sheetjs\Browser
 */
class BackendPageTest extends BrowserTestBase {

  /**
   * The test path for the backend configuration page.
   *
   * @var string
   */
  private $_testPath;

  /**
   * The submit button value.
   *
   * @var string
   */
  private $_submitValue;

  /**
   * The configuration factory object.
   *
   * @var object|null
   */
  private $_config_factory;

  /**
   * The modules required for the test.
   *
   * @var string[]
   */
  protected static $modules = ['drupal_sheetjs'];

  /**
   * The default theme for the test.
   *
   * @var string
   */
  protected $defaultTheme = 'classy';

  /**
   * Sets up the test environment.
   */
  public function setUp(): void {
    parent::setUp();
    $this->_testPath = 'admin/config/system/drupal-sheetjs';
    $this->_submitValue = 'Save configuration';
    $this->_config_factory = $this->container->get('config.factory');
  }

  /**
   * Tests access to the backend configuration page without login.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testAccessWithOutLogin() {
    $this->drupalGet($this->_testPath);
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests access to the backend configuration page with login.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAccessWithLogin() {
    $admin_user = $this->_createAdminuser();
    $this->drupalLogin($admin_user);
    $this->drupalGet($this->_testPath);
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Tests the configuration form submission.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testForm() {
    $admin_user = $this->_createAdminuser();
    $this->drupalLogin($admin_user);

    $this->_testFormTemplate('dest_table_id', 'dest_table_id');
    $this->_testFormTemplate('submit_element_id', 'submit_element_id');
    $this->_testFormTemplate('submit_full_element_id', 'submit_full_element_id');
    $this->_testFormTemplate('current_file_name', 'current_file_name');
    $this->_testFormTemplate('full_data_file_name', 'full_data_file_name');
    $this->_testFormTemplate('export_type', '20');
  }

  /**
   * Helper function to test form submission and configuration value.
   *
   * @param string $field_name
   *   The field name to test.
   * @param string $expected_value
   *   The expected value of the field.
   */
  private function _testFormTemplate(string $field_name, string $expected_value) {
    $this->drupalGet($this->_testPath);
    $this->assertSession()->elementExists('xpath', '//form[@id="drupal-sheetjs-settings"]');
    $this->submitForm([$field_name => $expected_value], $this->_submitValue);
    $value = $this->_config_factory->get('drupal_sheetjs.settings')->get($field_name);
    $this->assertSame($expected_value, $value);
  }

  /**
   * Helper function to create an admin user.
   *
   * @return \Drupal\user\Entity\User|false
   *   The admin user entity.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function _createAdminuser() {
    return $this->drupalCreateUser([
      'administer permissions',
      'administer users',
    ], 'administrator user01', TRUE);
  }

}
