<?php

namespace Drupal\Tests\drupal_sheetjs\Browser;

use Drupal\Tests\BrowserTestBase;
use Drupal\drupal_sheetjs\Helper\DrupalSheetJsHelper as Helper;

/**
 * Tests the front page access and functionality.
 *
 * @package Drupal\Tests\drupal_sheetjs\Browser
 */
class FrontPageTest extends BrowserTestBase {

  /**
   * The custom test path.
   *
   * @var string
   */
  private $_testCustomPath;

  /**
   * The default test path.
   *
   * @var string
   */
  private $_testDefaultPath;

  /**
   * The modules required for the test.
   *
   * @var string[]
   */
  protected static $modules = ['drupal_sheetjs'];

  /**
   * The default theme for the test.
   *
   * @var string
   */
  protected $defaultTheme = 'classy';

  /**
   * Sets up the test environment.
   */
  public function setUp(): void {
    parent::setUp();
    $this->_testDefaultPath = 'drupal-sheetjs/test';
    $this->_testCustomPath = 'drupal-sheetjs/test-custom';
  }

  /**
   * Tests access to the front page without login.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testAccessWithOutLogin() {
    // Test default path.
    $this->drupalGet($this->_testDefaultPath);
    $this->assertSession()->statusCodeEquals(403);

    // Test custom path.
    $this->drupalGet($this->_testCustomPath);
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests access to the front page with login.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Behat\Mink\Exception\ResponseTextException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAccessWithLogin() {
    $page = $this->getSession()->getPage();
    // Create admin user and login.
    $admin_user = $this->_createAdminuser();
    $this->drupalLogin($admin_user);

    $test_template = function (
      string $path = '',
      array $links = [],
      array $buttons = [],
      array $texts = []
    ) use ($page) {
      $this->drupalGet($path);
      $this->assertSession()->statusCodeEquals(200);
      foreach ($texts as $text) {
        $this->assertSession()->pageTextContains($text);
      }
      foreach ($links as $link) {
        $this->assertSession()->linkExists($link);
      }
      foreach ($buttons as $button) {
        $page->findLink($button)->click();
      }
    };

    // Test default path.
    $test_template(
      $this->_testDefaultPath,
      ['Export to XLSX!'],
      ['submitButtonId'],
      [
        'Drupal sheetjs script test default',
        'A1',
      ]
    );

    // Test custom path.
    $test_template(
      $this->_testCustomPath,
      ['Export to CSV!'],
      ['submitId'],
      [
        'Drupal sheetjs script test custom',
        'A1',
      ]
    );
  }

  /**
   * Tests access to the front page with login and click link.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testAccessWithLoginThenClickLink() {
    $page = $this->getSession()->getPage();

    // Create admin user and login.
    $admin_user = $this->_createAdminuser();
    $this->drupalLogin($admin_user);

    // Test default path.
    $this->drupalGet($this->_testDefaultPath);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->linkExists('Export full data to XLSX!');
    $page->findLink('submitFullButtonId')->click();

    // Test custom path.
    $this->drupalGet($this->_testCustomPath);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->linkExists('Export full data to CSV!');
    $page->findLink('submitFullId')->click();
  }

  /**
   * Tests access to the default page.
   */
  public function testAccessDefaultPage() {
    $idsArr = [
      Helper::$submit_element_id,
      Helper::$submit_full_element_id,
      Helper::$dest_table_id,
    ];

    $this->_testPageTypeTestTemplate($this->_testDefaultPath, $idsArr);
  }

  /**
   * Tests access to the custom page.
   */
  public function testAccessCustomPage() {
    $idsArr = [
      'submitId',
      'submitFullId',
      'tableId',
    ];

    $this->_testPageTypeTestTemplate($this->_testCustomPath, $idsArr);
  }

  /**
   * Helper function to test page type.
   *
   * @param string $path
   *   The path to test.
   * @param array $domIds
   *   The array of DOM element IDs.
   */
  private function _testPageTypeTestTemplate(string $path, array $domIds) {
    // Create admin user and login.
    $admin_user = $this->_createAdminuser();
    $this->drupalLogin($admin_user);

    // Test the page.
    $this->drupalGet($path);
    $this->assertSession()->statusCodeEquals(200);

    foreach ($domIds as $id) {
      $this->assertSession()->responseContains('id="' . $id . '"');
    }
  }

  /**
   * Helper function to create an admin user.
   *
   * @return \Drupal\user\Entity\User|false
   *   The admin user entity.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function _createAdminuser() {
    return $this->drupalCreateUser([
      'administer permissions',
      'administer users',
    ], 'administrator user01', TRUE);
  }

}
